@@ -1,7 +1,7 @@
 /*
     posix1e - a python module exposing the posix acl functions
 
-    Copyright (C) 2002-2009 Iustin Pop <iusty@k1024.org>
+    Copyright (C) 2002-2009, 2012 Iustin Pop <iusty@k1024.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -45,6 +45,10 @@
 #define PyInt_AsUnsignedLongMask PyLong_AsUnsignedLongMask
 #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
 #define PyInt_AS_LONG PyLong_AS_LONG
+#define MyString_ConcatAndDel PyUnicode_AppendAndDel
+#define MyString_FromFormat PyUnicode_FromFormat
+#define MyString_FromString PyUnicode_FromString
+#define MyString_FromStringAndSize PyUnicode_FromStringAndSize
 #else
 #define PyBytes_Check PyString_Check
 #define PyBytes_AS_STRING PyString_AS_STRING
@@ -52,6 +56,10 @@
 #define PyBytes_FromString PyString_FromString
 #define PyBytes_FromFormat PyString_FromFormat
 #define PyBytes_ConcatAndDel PyString_ConcatAndDel
+#define MyString_ConcatAndDel PyBytes_ConcatAndDel
+#define MyString_FromFormat PyBytes_FromFormat
+#define MyString_FromString PyBytes_FromString
+#define MyString_FromStringAndSize PyBytes_FromStringAndSize
 
 /* Python 2.6 already defines Py_TYPE */
 #ifndef Py_TYPE
@@ -208,9 +216,9 @@ static PyObject* ACL_str(PyObject *obj) {
     if(text == NULL) {
         return PyErr_SetFromErrno(PyExc_IOError);
     }
-    ret = PyBytes_FromString(text);
+    ret = MyString_FromString(text);
     if(acl_free(text) != 0) {
-        Py_DECREF(ret);
+        Py_XDECREF(ret);
         return PyErr_SetFromErrno(PyExc_IOError);
     }
     return ret;
@@ -218,29 +226,30 @@ static PyObject* ACL_str(PyObject *obj) {
 
 #ifdef HAVE_LINUX
 static char __to_any_text_doc__[] =
+  "to_any_text([prefix='', separator='n', options=0])\n"
   "Convert the ACL to a custom text format.\n"
   "\n"
-  "This method encapsulates the acl_to_any_text function. It allows a \n"
-  "customized text format to be generated for the ACL. See\n"
-  "acl_to_any_text(3) for more details.\n"
+  "This method encapsulates the ``acl_to_any_text()`` function.\n"
+  "It allows a customized text format to be generated for the ACL. See\n"
+  ":manpage:`acl_to_any_text(3)` for more details.\n"
   "\n"
-  "Parameters:\n"
-  "  - prefix: if given, this string will be prepended to all lines\n"
-  "  - separator: a single character (defaults to '\\n'); this will be\n"
-  "               user to separate the entries in the ACL\n"
-  "  - options: a bitwise combination of:\n"
-  "    -  TEXT_ABBREVIATE: use 'u' instead of 'user', 'g' instead of \n"
-  "                       'group', etc.\n"
-  "    -  TEXT_NUMERIC_IDS: User and group IDs are included as decimal\n"
-  "                         numbers instead of names\n"
-  "    -  TEXT_SOME_EFFECTIVE: Include comments denoting the effective\n"
-  "                            permissions when some are masked\n"
-  "    -  TEXT_ALL_EFFECTIVE: Include comments after all ACL entries\n"
-  "                           affected by an ACL_MASK entry\n"
-  "    -  TEXT_SMART_INDENT: Used in combination with the _EFFECTIVE\n"
-  "                          options, this will ensure that comments \n"
-  "                          are alligned to the fourth tab position\n"
-  "                          (assuming one tab equals eight spaces)\n"
+  ":param string prefix: if given, this string will be pre-pended to\n"
+  "   all lines\n"
+  ":param string separator: a single character (defaults to '\\n'); this will"
+    " be used to separate the entries in the ACL\n"
+  ":param options: a bitwise combination of:\n\n"
+  "    - :py:data:`TEXT_ABBREVIATE`: use 'u' instead of 'user', 'g' \n"
+  "      instead of 'group', etc.\n"
+  "    - :py:data:`TEXT_NUMERIC_IDS`: User and group IDs are included as\n"
+  "      decimal numbers instead of names\n"
+  "    - :py:data:`TEXT_SOME_EFFECTIVE`: Include comments denoting the\n"
+  "      effective permissions when some are masked\n"
+  "    - :py:data:`TEXT_ALL_EFFECTIVE`: Include comments after all ACL\n"
+  "      entries affected by an ACL_MASK entry\n"
+  "    - :py:data:`TEXT_SMART_INDENT`: Used in combination with the\n"
+  "      _EFFECTIVE options, this will ensure that comments are aligned\n"
+  "      to the fourth tab position (assuming one tab equals eight spaces)\n"
+  ":rtype: string\n"
   ;
 
 /* Converts the acl to a custom text format */
@@ -249,7 +258,7 @@ static PyObject* ACL_to_any_text(PyObject *obj, PyObject *args,
     char *text;
     ACL_Object *self = (ACL_Object*) obj;
     PyObject *ret;
-    char *arg_prefix = NULL;
+    const char *arg_prefix = NULL;
     char arg_separator = '\n';
     int arg_options = 0;
     static char *kwlist[] = {"prefix", "separator", "options", NULL};
@@ -264,7 +273,7 @@ static PyObject* ACL_to_any_text(PyObject *obj, PyObject *args,
     }
     ret = PyBytes_FromString(text);
     if(acl_free(text) != 0) {
-        Py_DECREF(ret);
+        Py_XDECREF(ret);
         return PyErr_SetFromErrno(PyExc_IOError);
     }
     return ret;
@@ -274,18 +283,20 @@ static char __check_doc__[] =
     "Check the ACL validity.\n"
     "\n"
     "This is a non-portable, Linux specific extension that allow more\n"
-    "information to be retrieved in case an ACL is not valid than the\n"
-    "validate() method.\n"
+    "information to be retrieved in case an ACL is not valid than via the\n"
+    ":py:func:`valid` method.\n"
     "\n"
     "This method will return either False (the ACL is valid), or a tuple\n"
     "with two elements. The first element is one of the following\n"
-    "constants:\n"
-    "  - ACL_MULTI_ERROR: The ACL contains multiple entries that have a\n"
-    "                     tag type that may occur at most once\n"
-    "  - ACL_DUPLICATE_ERROR: The ACL contains multiple ACL_USER or \n"
-    "                         ACL_GROUP entries  with the same ID\n"
-    "  - ACL_MISS_ERROR: A required entry is missing\n"
-    "  - ACL_ENTRY_ERROR: The ACL contains an invalid entry tag type\n"
+    "constants:\n\n"
+    "  - :py:data:`ACL_MULTI_ERROR`: The ACL contains multiple entries that\n"
+    "    have a tag type that may occur at most once\n"
+    "  - :py:data:`ACL_DUPLICATE_ERROR`: The ACL contains multiple \n"
+    "    :py:data:`ACL_USER` or :py:data:`ACL_GROUP` entries with the\n"
+    "    same ID\n"
+    "  - :py:data:`ACL_MISS_ERROR`: A required entry is missing\n"
+    "  - :py:data:`ACL_ENTRY_ERROR`: The ACL contains an invalid entry\n"
+    "    tag type\n"
     "\n"
     "The second element of the tuple is the index of the entry that is\n"
     "invalid (in the same order as by iterating over the ACL entry)\n"
@@ -300,10 +311,9 @@ static PyObject* ACL_check(PyObject* obj, PyObject* args) {
     if((result = acl_check(self->acl, &eindex)) == -1)
         return PyErr_SetFromErrno(PyExc_IOError);
     if(result == 0) {
-        Py_INCREF(Py_False);
-        return Py_False;
+        Py_RETURN_FALSE;
     }
-    return PyTuple_Pack(2, PyInt_FromLong(result), PyInt_FromLong(eindex));
+    return Py_BuildValue("(ii)", result, eindex);
 }
 
 /* Implementation of the rich compare for ACLs */
@@ -345,7 +355,9 @@ static char __equiv_mode_doc__[] =
     "This is a non-portable, Linux specific extension that checks\n"
     "if the ACL is a basic ACL and returns the corresponding mode.\n"
     "\n"
-    "An IOerror exception will be raised if the ACL is an extended ACL\n"
+    ":rtype: integer\n"
+    ":raise IOError: An IOerror exception will be raised if the ACL is\n"
+    "    an extended ACL.\n"
     ;
 
 /* The acl_equiv_mode method */
@@ -368,16 +380,16 @@ static int ACL_nocmp(PyObject* o1, PyObject* o2) {
 
 /* Custom methods */
 static char __applyto_doc__[] =
+    "applyto(item[, flag=ACL_TYPE_ACCESS])\n"
     "Apply the ACL to a file or filehandle.\n"
     "\n"
-    "Parameters:\n"
-    "  - either a filename or a file-like object or an integer; this\n"
-    "    represents the filesystem object on which to act\n"
-    "  - optional flag representing the type of ACL to set, either\n"
-    "    ACL_TYPE_ACCESS (default) or ACL_TYPE_DEFAULT\n"
+    ":param item: either a filename or a file-like object or an integer;\n"
+    "    this represents the filesystem object on which to act\n"
+    ":param flag: optional flag representing the type of ACL to set, either\n"
+    "    :py:data:`ACL_TYPE_ACCESS` (default) or :py:data:`ACL_TYPE_DEFAULT`\n"
     ;
 
-/* Applyes the ACL to a file */
+/* Applies the ACL to a file */
 static PyObject* ACL_applyto(PyObject* obj, PyObject* args) {
     ACL_Object *self = (ACL_Object*) obj;
     PyObject *myarg;
@@ -385,7 +397,7 @@ static PyObject* ACL_applyto(PyObject* obj, PyObject* args) {
     int nret;
     int fd;
 
-    if (!PyArg_ParseTuple(args, "O|i", &myarg, &type))
+    if (!PyArg_ParseTuple(args, "O|I", &myarg, &type))
         return NULL;
 
     if(PyBytes_Check(myarg)) {
@@ -420,25 +432,32 @@ static char __valid_doc__[] =
     "Test the ACL for validity.\n"
     "\n"
     "This method tests the ACL to see if it is a valid ACL\n"
-    "in terms of the filesystem. More precisely, it checks that:\n"
+    "in terms of the file-system. More precisely, it checks that:\n"
     "\n"
     "The ACL contains exactly one entry with each of the\n"
-    "ACL_USER_OBJ, ACL_GROUP_OBJ, and ACL_OTHER tag types. Entries\n"
-    "with ACL_USER and ACL_GROUP tag types may appear zero or more\n"
-    "times in an ACL. An ACL that contains entries of ACL_USER or\n"
-    "ACL_GROUP tag types must contain exactly one entry of the \n"
-    "ACL_MASK tag type. If an ACL contains no entries of\n"
-    "ACL_USER or ACL_GROUP tag types, the ACL_MASK entry is optional.\n"
+    ":py:data:`ACL_USER_OBJ`, :py:data:`ACL_GROUP_OBJ`, and \n"
+    ":py:data:`ACL_OTHER` tag types. Entries\n"
+    "with :py:data:`ACL_USER` and :py:data:`ACL_GROUP` tag types may\n"
+    "appear zero or more\n"
+    "times in an ACL. An ACL that contains entries of :py:data:`ACL_USER` or\n"
+    ":py:data:`ACL_GROUP` tag types must contain exactly one entry of the \n"
+    ":py:data:`ACL_MASK` tag type. If an ACL contains no entries of\n"
+    ":py:data:`ACL_USER` or :py:data:`ACL_GROUP` tag types, the\n"
+    ":py:data:`ACL_MASK` entry is optional.\n"
     "\n"
     "All user ID qualifiers must be unique among all entries of\n"
-    "the ACL_USER tag type, and all group IDs must be unique among all\n"
-    "entries of ACL_GROUP tag type.\n"
+    "the :py:data:`ACL_USER` tag type, and all group IDs must be unique\n"
+    "among all entries of :py:data:`ACL_GROUP` tag type.\n"
     "\n"
     "The method will return 1 for a valid ACL and 0 for an invalid one.\n"
-    "This has been chosen because the specification for acl_valid in\n"
-    "the POSIX.1e standard documents only one possible value for errno\n"
+    "This has been chosen because the specification for\n"
+    ":manpage:`acl_valid(3)`\n"
+    "in the POSIX.1e standard documents only one possible value for errno\n"
     "in case of an invalid ACL, so we can't differentiate between\n"
     "classes of errors. Other suggestions are welcome.\n"
+    "\n"
+    ":return: 0 or 1\n"
+    ":rtype: integer\n"
     ;
 
 /* Checks the ACL for validity */
@@ -446,11 +465,9 @@ static PyObject* ACL_valid(PyObject* obj, PyObject* args) {
     ACL_Object *self = (ACL_Object*) obj;
 
     if(acl_valid(self->acl) == -1) {
-        Py_INCREF(Py_False);
-        return Py_False;
+        Py_RETURN_FALSE;
     } else {
-        Py_INCREF(Py_True);
-        return Py_True;
+        Py_RETURN_TRUE;
     }
 }
 
@@ -547,11 +564,12 @@ static PyObject* ACL_iternext(PyObject *obj) {
 }
 
 static char __ACL_delete_entry_doc__[] =
+    "delete_entry(entry)\n"
     "Deletes an entry from the ACL.\n"
     "\n"
-    "Note: Only with level 2\n"
-    "Parameters:\n"
-    "  - the Entry object which should be deleted; note that after\n"
+    ".. note:: Only available with level 2.\n"
+    "\n"
+    ":param entry: the Entry object which should be deleted; note that after\n"
     "    this function is called, that object is unusable any longer\n"
     "    and should be deleted\n"
     ;
@@ -576,12 +594,14 @@ static char __ACL_calc_mask_doc__[] =
     "Compute the file group class mask.\n"
     "\n"
     "The calc_mask() method calculates and sets the permissions \n"
-    "associated with the ACL_MASK Entry of the ACL.\n"
+    "associated with the :py:data:`ACL_MASK` Entry of the ACL.\n"
     "The value of the new permissions is the union of the permissions \n"
-    "granted by all entries of tag type ACL_GROUP, ACL_GROUP_OBJ, or \n"
-    "ACL_USER.  If the ACL already contains an ACL_MASK entry, its \n"
-    "permissions are overwritten; if it does not contain an ACL_MASK \n"
-    "Entry, one is added.\n"
+    "granted by all entries of tag type :py:data:`ACL_GROUP`, \n"
+    ":py:data:`ACL_GROUP_OBJ`, or \n"
+    ":py:data:`ACL_USER`. If the ACL already contains an :py:data:`ACL_MASK`\n"
+    "entry, its \n"
+    "permissions are overwritten; if it does not contain an \n"
+    ":py:data:`ACL_MASK` Entry, one is added.\n"
     "\n"
     "The order of existing entries in the ACL is undefined after this \n"
     "function.\n"
@@ -600,13 +620,17 @@ static PyObject* ACL_calc_mask(PyObject *obj, PyObject *args) {
 }
 
 static char __ACL_append_doc__[] =
+    "append([entry])\n"
     "Append a new Entry to the ACL and return it.\n"
     "\n"
     "This is a convenience function to create a new Entry \n"
     "and append it to the ACL.\n"
     "If a parameter of type Entry instance is given, the \n"
     "entry will be a copy of that one (as if copied with \n"
-    "Entry.copy()), otherwise, the new entry will be empty.\n"
+    ":py:func:`Entry.copy`), otherwise, the new entry will be empty.\n"
+    "\n"
+    ":rtype: :py:class:`Entry`\n"
+    ":returns: the newly created entry\n"
     ;
 
 /* Convenience method to create a new Entry */
@@ -621,8 +645,10 @@ static PyObject* ACL_append(PyObject *obj, PyObject *args) {
         return NULL;
     }
 
-    if (!PyArg_ParseTuple(args, "|O!", &Entry_Type, &oldentry))
+    if (!PyArg_ParseTuple(args, "|O!", &Entry_Type, &oldentry)) {
+        Py_DECREF(newentry);
         return NULL;
+    }
 
     nret = acl_create_entry(&self->acl, &newentry->entry);
     if(nret == -1) {
@@ -720,30 +746,40 @@ static PyObject* Entry_str(PyObject *obj) {
         qualifier = 0;
     }
 
-    format = PyBytes_FromString("ACL entry for ");
+    format = MyString_FromString("ACL entry for ");
     if(format == NULL)
         return NULL;
-    if(tag == ACL_UNDEFINED_TAG) {
-        kind = PyBytes_FromString("undefined type");
-    } else if(tag == ACL_USER_OBJ) {
-        kind = PyBytes_FromString("the owner");
-    } else if(tag == ACL_GROUP_OBJ) {
-        kind = PyBytes_FromString("the group");
-    } else if(tag == ACL_OTHER) {
-        kind = PyBytes_FromString("the others");
-    } else if(tag == ACL_USER) {
-        kind = PyBytes_FromFormat("user with uid %d", qualifier);
-    } else if(tag == ACL_GROUP) {
-        kind = PyBytes_FromFormat("group with gid %d", qualifier);
-    } else if(tag == ACL_MASK) {
-        kind = PyBytes_FromString("the mask");
-    } else {
-        kind = PyBytes_FromString("UNKNOWN_TAG_TYPE!");
+    switch(tag) {
+    case ACL_UNDEFINED_TAG:
+        kind = MyString_FromString("undefined type");
+        break;
+    case ACL_USER_OBJ:
+        kind = MyString_FromString("the owner");
+        break;
+    case ACL_GROUP_OBJ:
+        kind = MyString_FromString("the group");
+        break;
+    case ACL_OTHER:
+        kind = MyString_FromString("the others");
+        break;
+    case ACL_USER:
+        kind = MyString_FromFormat("user with uid %d", qualifier);
+        break;
+    case ACL_GROUP:
+        kind = MyString_FromFormat("group with gid %d", qualifier);
+        break;
+    case ACL_MASK:
+        kind = MyString_FromString("the mask");
+        break;
+    default:
+        kind = MyString_FromString("UNKNOWN_TAG_TYPE!");
+        break;
     }
-    if (kind == NULL)
+    if (kind == NULL) {
+        Py_DECREF(format);
         return NULL;
-    PyBytes_ConcatAndDel(&format, kind);
-    Py_DECREF(format);
+    }
+    MyString_ConcatAndDel(&format, kind);
     return format;
 }
 
@@ -857,6 +893,7 @@ static PyObject* Entry_get_permset(PyObject *obj, void* arg) {
     ps = (Permset_Object*)p;
     if(acl_get_permset(self->entry, &ps->permset) == -1) {
         PyErr_SetFromErrno(PyExc_IOError);
+        Py_DECREF(p);
         return NULL;
     }
     ps->parent_entry = obj;
@@ -883,15 +920,16 @@ static int Entry_set_permset(PyObject* obj, PyObject* value, void* arg) {
 }
 
 static char __Entry_copy_doc__[] =
-    "Copy an ACL entry.\n"
+    "copy(src)\n"
+    "Copies an ACL entry.\n"
     "\n"
     "This method sets all the parameters to those of another\n"
-    "entry, even one of another's ACL\n"
-    "Parameters:\n"
-    " - src, instance of type Entry\n"
+    "entry (either of the same ACL or belonging to another ACL).\n"
+    "\n"
+    ":param Entry src: instance of type Entry\n"
     ;
 
-/* Sets all the entry parameters to another's entry */
+/* Sets all the entry parameters to another entry */
 static PyObject* Entry_copy(PyObject *obj, PyObject *args) {
     Entry_Object *self = (Entry_Object*)obj;
     Entry_Object *other;
@@ -967,11 +1005,11 @@ static PyObject* Permset_str(PyObject *obj) {
     pstr[0] = get_perm(self->permset, ACL_READ) ? 'r' : '-';
     pstr[1] = get_perm(self->permset, ACL_WRITE) ? 'w' : '-';
     pstr[2] = get_perm(self->permset, ACL_EXECUTE) ? 'x' : '-';
-    return PyBytes_FromStringAndSize(pstr, 3);
+    return MyString_FromStringAndSize(pstr, 3);
 }
 
 static char __Permset_clear_doc__[] =
-    "Clear all permissions from the permission set.\n"
+    "Clears all permissions from the permission set.\n"
     ;
 
 /* Clears all permissions from the permset */
@@ -990,11 +1028,9 @@ static PyObject* Permset_get_right(PyObject *obj, void* arg) {
     Permset_Object *self = (Permset_Object*) obj;
 
     if(get_perm(self->permset, *(acl_perm_t*)arg)) {
-        Py_INCREF(Py_True);
-        return Py_True;
+        Py_RETURN_TRUE;
     } else {
-        Py_INCREF(Py_False);
-        return Py_False;
+        Py_RETURN_FALSE;
     }
 }
 
@@ -1021,19 +1057,17 @@ static int Permset_set_right(PyObject* obj, PyObject* value, void* arg) {
 }
 
 static char __Permset_add_doc__[] =
+    "add(perm)\n"
     "Add a permission to the permission set.\n"
     "\n"
-    "The add() function adds the permission contained in \n"
+    "This function adds the permission contained in \n"
     "the argument perm to the permission set.  An attempt \n"
     "to add a permission that is already contained in the \n"
     "permission set is not considered an error.\n"
     "\n"
-    "Parameters:\n\n"
-    "  - perm: a permission (ACL_WRITE, ACL_READ, ACL_EXECUTE, ...)\n"
-    "\n"
-    "Return value: None\n"
-    "\n"
-    "Can raise: IOError\n"
+    ":param perm: a permission (:py:data:`ACL_WRITE`, :py:data:`ACL_READ`,\n"
+    "   :py:data:`ACL_EXECUTE`, ...)\n"
+    ":raises IOError: in case the argument is not a valid descriptor\n"
     ;
 
 static PyObject* Permset_add(PyObject* obj, PyObject* args) {
@@ -1052,17 +1086,17 @@ static PyObject* Permset_add(PyObject* obj, PyObject* args) {
 }
 
 static char __Permset_delete_doc__[] =
+    "delete(perm)\n"
     "Delete a permission from the permission set.\n"
     "\n"
-    "The delete() function deletes the permission contained in \n"
-    "the argument perm from the permission set.  An attempt \n"
+    "This function deletes the permission contained in \n"
+    "the argument perm from the permission set. An attempt \n"
     "to delete a permission that is not contained in the \n"
     "permission set is not considered an error.\n"
-    "Parameters:\n\n"
-    "  - perm a permission (ACL_WRITE, ACL_READ, ACL_EXECUTE, ...)\n"
-    "Return value: None\n"
     "\n"
-    "Can raise: IOError\n"
+    ":param perm: a permission (:py:data:`ACL_WRITE`, :py:data:`ACL_READ`,\n"
+    "   :py:data:`ACL_EXECUTE`, ...)\n"
+    ":raises IOError: in case the argument is not a valid descriptor\n"
     ;
 
 static PyObject* Permset_delete(PyObject* obj, PyObject* args) {
@@ -1081,15 +1115,16 @@ static PyObject* Permset_delete(PyObject* obj, PyObject* args) {
 }
 
 static char __Permset_test_doc__[] =
+    "test(perm)\n"
     "Test if a permission exists in the permission set.\n"
     "\n"
-    "The test() function tests if the permission contained in \n"
-    "the argument perm exits the permission set.\n"
-    "Parameters:\n\n"
-    "  - perm a permission (ACL_WRITE, ACL_READ, ACL_EXECUTE, ...)\n"
-    "Return value: Boolean\n"
+    "The test() function tests if the permission represented by\n"
+    "the argument perm exists in the permission set.\n"
     "\n"
-    "Can raise: IOError\n"
+    ":param perm: a permission (:py:data:`ACL_WRITE`, :py:data:`ACL_READ`,\n"
+    "   :py:data:`ACL_EXECUTE`, ...)\n"
+    ":rtype: Boolean\n"
+    ":raises IOError: in case the argument is not a valid descriptor\n"
     ;
 
 static PyObject* Permset_test(PyObject* obj, PyObject* args) {
@@ -1105,11 +1140,9 @@ static PyObject* Permset_test(PyObject* obj, PyObject* args) {
         return PyErr_SetFromErrno(PyExc_IOError);
 
     if(ret) {
-        Py_INCREF(Py_True);
-        return Py_True;
+        Py_RETURN_TRUE;
     } else {
-        Py_INCREF(Py_False);
-        return Py_False;
+        Py_RETURN_FALSE;
     }
 }
 
@@ -1118,24 +1151,25 @@ static PyObject* Permset_test(PyObject* obj, PyObject* args) {
 static char __ACL_Type_doc__[] =
     "Type which represents a POSIX ACL\n"
     "\n"
-    "Parameters (only one keword parameter should be provided):\n"
-    "  - file=\"...\", meaning create ACL representing\n"
-    "    the access ACL of that file\n"
-    "  - filedef=\"...\", meaning create ACL representing\n"
-    "    the default ACL of that directory\n"
-    "  - fd=<int>, meaning create ACL representing\n"
-    "    the access ACL of that file descriptor\n"
-    "  - text=\"...\", meaning create ACL from a \n"
+    ".. note:: only one keyword parameter should be provided\n"
+    "\n"
+    ":param string file: creates an ACL representing\n"
+    "    the access ACL of the specified file\n"
+    ":param string filedef: creates an ACL representing\n"
+    "    the default ACL of the given directory\n"
+    ":param int fd: creates an ACL representing\n"
+    "    the access ACL of the given file descriptor\n"
+    ":param string text: creates an ACL from a \n"
     "    textual description\n"
-    "  - acl=<ACL instance>, meaning create a copy\n"
-    "    of an existing ACL instance\n"
-    "  - mode=<int>, meaning create an ACL from a numeric mode\n"
+    ":param ACL acl: creates a copy of an existing ACL instance\n"
+    ":param int mode: creates an ACL from a numeric mode\n"
     "    (e.g. mode=0644) (this is valid only when the C library\n"
     "    provides the acl_from_mode call)\n"
     "\n"
-    "If no parameters are passed, create an empty ACL; this\n"
+    "If no parameters are passed, an empty ACL will be created; this\n"
     "makes sense only when your OS supports ACL modification\n"
-    "(i.e. it implements full POSIX.1e support)\n"
+    "(i.e. it implements full POSIX.1e support), otherwise the ACL won't\n"
+    "be useful.\n"
     ;
 
 /* ACL type methods */
@@ -1231,21 +1265,21 @@ static char __Entry_tagtype_doc__[] =
     "The tag type of the current entry\n"
     "\n"
     "This is one of:\n"
-    " - ACL_UNDEFINED_TAG\n"
-    " - ACL_USER_OBJ\n"
-    " - ACL_USER\n"
-    " - ACL_GROUP_OBJ\n"
-    " - ACL_GROUP\n"
-    " - ACL_MASK\n"
-    " - ACL_OTHER\n"
+    " - :py:data:`ACL_UNDEFINED_TAG`\n"
+    " - :py:data:`ACL_USER_OBJ`\n"
+    " - :py:data:`ACL_USER`\n"
+    " - :py:data:`ACL_GROUP_OBJ`\n"
+    " - :py:data:`ACL_GROUP`\n"
+    " - :py:data:`ACL_MASK`\n"
+    " - :py:data:`ACL_OTHER`\n"
     ;
 
 static char __Entry_qualifier_doc__[] =
     "The qualifier of the current entry\n"
     "\n"
-    "If the tag type is ACL_USER, this should be a user id.\n"
-    "If the tag type if ACL_GROUP, this should be a group id.\n"
-    "Else, it doesn't matter.\n"
+    "If the tag type is :py:data:`ACL_USER`, this should be a user id.\n"
+    "If the tag type if :py:data:`ACL_GROUP`, this should be a group id.\n"
+    "Else it doesn't matter.\n"
     ;
 
 static char __Entry_parent_doc__[] =
@@ -1277,6 +1311,7 @@ static char __Entry_Type_doc__[] =
     "  >>> e = myACL.append() # another way for doing the same thing\n"
     "\n"
     "or by:\n"
+    "\n"
     "  >>> for entry in myACL:\n"
     "  ...     print entry\n"
     "\n"
@@ -1341,27 +1376,30 @@ static PyMethodDef Permset_methods[] = {
 };
 
 static char __Permset_execute_doc__[] =
-    "Execute permsission\n"
+    "Execute permission property\n"
     "\n"
-    "This is a convenience method of access; the \n"
+    "This is a convenience method of retrieving and setting the execute\n"
+    "permission in the permission set; the \n"
     "same effect can be achieved using the functions\n"
     "add(), test(), delete(), and those can take any \n"
     "permission defined by your platform.\n"
     ;
 
 static char __Permset_read_doc__[] =
-    "Read permsission\n"
+    "Read permission property\n"
     "\n"
-    "This is a convenience method of access; the \n"
+    "This is a convenience method of retrieving and setting the read\n"
+    "permission in the permission set; the \n"
     "same effect can be achieved using the functions\n"
     "add(), test(), delete(), and those can take any \n"
     "permission defined by your platform.\n"
     ;
 
 static char __Permset_write_doc__[] =
-    "Write permsission\n"
+    "Write permission property\n"
     "\n"
-    "This is a convenience method of access; the \n"
+    "This is a convenience method of retrieving and setting the write\n"
+    "permission in the permission set; the \n"
     "same effect can be achieved using the functions\n"
     "add(), test(), delete(), and those can take any \n"
     "permission defined by your platform.\n"
@@ -1445,14 +1483,14 @@ static PyTypeObject Permset_Type = {
 /* Module methods */
 
 static char __deletedef_doc__[] =
+    "delete_default(path)\n"
     "Delete the default ACL from a directory.\n"
     "\n"
-    "This function deletes the default ACL associated with \n"
+    "This function deletes the default ACL associated with\n"
     "a directory (the ACL which will be ANDed with the mode\n"
     "parameter to the open, creat functions).\n"
-    "Parameters:\n"
-    "  - a string representing the directory whose default ACL\n"
-    "    should be deleted\n"
+    "\n"
+    ":param string path: the directory whose default ACL should be deleted\n"
     ;
 
 /* Deletes the default ACL from a directory */
@@ -1474,11 +1512,11 @@ static PyObject* aclmodule_delete_default(PyObject* obj, PyObject* args) {
 
 #ifdef HAVE_LINUX
 static char __has_extended_doc__[] =
-    "Check if a file or filehandle has an extended ACL.\n"
+    "has_extended(item)\n"
+    "Check if a file or file handle has an extended ACL.\n"
     "\n"
-    "Parameter:\n"
-    "  - either a filename or a file-like object or an integer; this\n"
-    "    represents the filesystem object on which to act\n"
+    ":param item: either a file name or a file-like object or an integer;\n"
+    "  it represents the file-system object on which to act\n"
     ;
 
 /* Check for extended ACL a file or fd */
@@ -1531,11 +1569,12 @@ static PyMethodDef aclmodule_methods[] = {
 
 static char __posix1e_doc__[] =
     "POSIX.1e ACLs manipulation\n"
+    "==========================\n"
     "\n"
     "This module provides support for manipulating POSIX.1e ACLS\n"
     "\n"
     "Depending on the operating system support for POSIX.1e, \n"
-    "the ACL type will have more or less capabilities:\n"
+    "the ACL type will have more or less capabilities:\n\n"
     "  - level 1, only basic support, you can create\n"
     "    ACLs from files and text descriptions;\n"
     "    once created, the type is immutable\n"
@@ -1546,12 +1585,13 @@ static char __posix1e_doc__[] =
     "to acl_entry_t (the Entry type), acl_permset_t (the Permset type).\n"
     "\n"
     "The existence of level 2 support and other extensions can be\n"
-    "checked by the constants:\n"
-    "  - HAS_ACL_ENTRY for level 2 and the Entry/Permset classes\n"
-    "  - HAS_ACL_FROM_MODE for ACL(mode=...) usage\n"
-    "  - HAS_ACL_CHECK for the ACL().check function\n"
-    "  - HAS_EXTENDED_CHECK for the module-level has_extended function\n"
-    "  - HAS_EQUIV_MODE for the ACL().equiv_mode method\n"
+    "checked by the constants:\n\n"
+    "  - :py:data:`HAS_ACL_ENTRY` for level 2 and the Entry/Permset classes\n"
+    "  - :py:data:`HAS_ACL_FROM_MODE` for ``ACL(mode=...)`` usage\n"
+    "  - :py:data:`HAS_ACL_CHECK` for the :py:func:`ACL.check` function\n"
+    "  - :py:data:`HAS_EXTENDED_CHECK` for the module-level\n"
+    "    :py:func:`has_extended` function\n"
+    "  - :py:data:`HAS_EQUIV_MODE` for the :py:func:`ACL.equiv_mode` method\n"
     "\n"
     "Example:\n"
     "\n"
@@ -1575,6 +1615,52 @@ static char __posix1e_doc__[] =
     "other::---\n"
     ">>>\n"
     "\n"
+    ".. py:data:: ACL_USER\n\n"
+    "   Denotes a specific user entry in an ACL.\n"
+    "\n"
+    ".. py:data:: ACL_USER_OBJ\n\n"
+    "   Denotes the user owner entry in an ACL.\n"
+    "\n"
+    ".. py:data:: ACL_GROUP\n\n"
+    "   Denotes the a group entry in an ACL.\n"
+    "\n"
+    ".. py:data:: ACL_GROUP_OBJ\n\n"
+    "   Denotes the group owner entry in an ACL.\n"
+    "\n"
+    ".. py:data:: ACL_OTHER\n\n"
+    "   Denotes the 'others' entry in an ACL.\n"
+    "\n"
+    ".. py:data:: ACL_MASK\n\n"
+    "   Denotes the mask entry in an ACL, representing the maximum\n"
+    "   access granted other users, the owner group and other groups.\n"
+    "\n"
+    ".. py:data:: ACL_UNDEFINED_TAG\n\n"
+    "   An undefined tag in an ACL.\n"
+    "\n"
+    ".. py:data:: ACL_READ\n\n"
+    "   Read permission in a permission set.\n"
+    "\n"
+    ".. py:data:: ACL_WRITE\n\n"
+    "   Write permission in a permission set.\n"
+    "\n"
+    ".. py:data:: ACL_EXECUTE\n\n"
+    "   Execute permission in a permission set.\n"
+    "\n"
+    ".. py:data:: HAS_ACL_ENTRY\n\n"
+    "   denotes support for level 2 and the Entry/Permset classes\n"
+    "\n"
+    ".. py:data:: HAS_ACL_FROM_MODE\n\n"
+    "   denotes support for building an ACL from an octal mode\n"
+    "\n"
+    ".. py:data:: HAS_ACL_CHECK\n\n"
+    "   denotes support for extended checks of an ACL's validity\n"
+    "\n"
+    ".. py:data:: HAS_EXTENDED_CHECK\n\n"
+    "   denotes support for checking whether an ACL is basic or extended\n"
+    "\n"
+    ".. py:data:: HAS_EQUIV_MODE\n\n"
+    "   denotes support for the equiv_mode function\n"
+    "\n"
     ;
 
 #ifdef IS_PY3K
